#include <stdio.h>
#include <limits.h>

#include "globaldefn.h"

/* kludge, runtime.h does not check for double inclusion */
#if !defined(SP_TYPE_VARIABLE)
#	define PROTO
#	include "/usr/local/lib/sicstus2.1/Runtime/runtime.h"
#endif

extern int getopt FN_PROTOTYPE((int argc, char **argv, char *optString));
extern char *optarg;
extern int optind, opterr;

static void Usage FN_PROTOTYPE((void));

static char *progName;

int
main(int argc, char *argv[])
{
	char *agent, *object, *goal, *module;
	int retValue, opt;
	SP_term SP_agent, SP_flag;
	SP_pred_ref SP_goal;


	progName = argv[0];
	opterr = FALSE;

	while ((opt = getopt(argc, argv, "su")) != -1)
	{
		switch(opt)
		{
		case 'u':
			Usage();
			exit(0);
			break;
		case '?':
			Usage();
			exit(1);
			break;
		default:
			/* Something wrong with the list of valid options */
			fprintf(stderr, 
				"%s: Internal error. (%s/%d)\n", progName, __FILE__, __LINE__);
			Usage();
			exit(1);
			break;
		}
	}

	if (argc - optind != 3 && argc - optind != 4)
	{
		Usage();
		exit(1);
	}

	agent = argv[optind];
	object = argv[optind+1];
	goal = argv[optind+2];
	if (argc - optind == 4)
		module = argv[optind+3];
	else
		module = NULL;

	if (! SP_initialize())
	{
		fprintf(stderr,"%s: SP_initialize failed!\n", progName);
		exit(1);
	}

	if (! SP_load(object))
	{
		fprintf(stderr,"%s: %s could not be found!\n", progName, object);
		exit(1);
	}

	if (! (SP_goal = SP_predicate(goal, 2, module)) )
	{
		fprintf(stderr,"%s: %s%s%s/2 is not found in object %s\n", progName, 
			(module ? module : ""), (module ? ":" : ""), goal, object);
		exit(1);
	}

	SP_show_term(&SP_agent);
	SP_put_string(&SP_agent, agent);
	SP_show_term(&SP_flag);
	SP_put_string(&SP_flag, "runtime");
	retValue = SP_query(SP_goal, &SP_agent, &SP_flag);
	SP_hide_term(&SP_agent);
	SP_hide_term(&SP_flag);

	return 0;
}

static void 
Usage(void)
{
	fprintf(stderr,
		"Usage: %s [-u] <agentname> <prolog object> <goal> [module]\n",
			progName);
	fprintf(stderr,
		"       u: This message\n\n");
	fprintf(stderr,
		"Note: goal should be a predicate of arity 2.\n");
	fprintf(stderr,
		"      Both the arguments will be instantiated to atoms.\n");
	fprintf(stderr,
		"      agentname will be passed as the first argument.\n");
	fprintf(stderr,
		"      'runtime' will be passed as the second argument.\n");
}
